package org.fuseri.moduleexercise.answer;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.persistence.EntityNotFoundException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.fuseri.model.dto.exercise.AnswerCreateDto;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.AnswersCreateDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

/**
 * Represent a REST API controller for answers
 * Handle HTTP requests related to answers
 */
@RestController
@RequestMapping("/answers")
public class AnswerController {

    private final AnswerFacade facade;

    @Autowired
    public AnswerController(AnswerFacade facade) {
        this.facade = facade;
    }

    /**
     * Create a new answer for the given question ID
     *
     * @param dto the AnswerCreateDto object containing information about the answer to create
     * @return a ResponseEntity containing an AnswerDto object representing the newly created answer, or a 404 Not Found response
     * if the question with the specified ID in dto was not found
     */
    @Operation(summary = "Create new answers for question", description = "Creates new answers for question.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "201", description = "Answers created successfully."),
            @ApiResponse(responseCode = "400", description = "Invalid input.")
    })
    @PostMapping
    public ResponseEntity<List<AnswerDto>> createMultiple(@Valid @RequestBody AnswersCreateDto dto) {
        try {
            return ResponseEntity.status(HttpStatus.CREATED).body(facade.createMultiple(dto));
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }

    /**
     * Update an answer
     *
     * @param id  of answer to update
     * @param dto dto with updated answer information
     * @return A ResponseEntity with an AnswerDto object representing the updated answer on an HTTP status code of 200 if the update was successful.
     * or a NOT_FOUND response if the answer ID is invalid
     */
    @Operation(summary = "Update an answer", description = "Updates an answer with the specified ID.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Answer with the specified ID updated successfully."),
            @ApiResponse(responseCode = "400", description = "Invalid input."),
            @ApiResponse(responseCode = "404", description = "Answer with the specified ID was not found.")
    })
    @PutMapping("/{id}")
    public ResponseEntity<AnswerDto> update(@NotNull @PathVariable long id, @Valid @RequestBody AnswerCreateDto dto) {
        try {
            return ResponseEntity.ok(facade.update(id, dto));
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }

    /**
     * Delete answer with the specified ID
     *
     * @param id of answer to delete
     * @throws ResponseStatusException if answer with specified id does not exist
     */
    @Operation(summary = "Delete an answer with specified ID", description = "Deletes an answer with the specified ID.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Answer with the specified ID deleted successfully."),
            @ApiResponse(responseCode = "404", description = "Answer with the specified ID was not found.")
    })
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> delete(@NotNull @PathVariable long id) {
        try {
            facade.delete(id);
            return ResponseEntity.noContent().build();
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }

//    @TestOnly
    @DeleteMapping("/reset")
    public void resetTable() {
        facade.resetTable();
    }

}
