package org.fuseri.moduleexercise.question;

import jakarta.transaction.Transactional;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.QuestionCreateDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.fuseri.model.dto.exercise.QuestionUpdateDto;
import org.fuseri.moduleexercise.answer.Answer;
import org.fuseri.moduleexercise.answer.AnswerMapper;
import org.fuseri.moduleexercise.answer.AnswerService;
import org.fuseri.moduleexercise.exercise.Exercise;
import org.fuseri.moduleexercise.exercise.ExerciseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;

/**
 * Represent facade for managing questions
 * Provide simplified interface for manipulating with questions
 */
@Transactional
@Service
public class QuestionFacade {
    private final QuestionService questionService;
    private final ExerciseService exerciseService;
    private final AnswerService answerService;
    private final QuestionMapper questionMapper;
    private final AnswerMapper answerMapper;

    @Autowired
    public QuestionFacade(
            QuestionService questionService, ExerciseService exerciseService,
            AnswerService answerService, QuestionMapper questionMapper,
            AnswerMapper answerMapper) {
        this.questionService = questionService;
        this.exerciseService = exerciseService;
        this.answerService = answerService;
        this.questionMapper = questionMapper;
        this.answerMapper = answerMapper;
    }

    /**
     * Find a question by ID.
     *
     * @param id the ID of the question to find
     * @return a QuestionDto object representing the found question
     */
    public QuestionDto find(long id) {
        var a = questionService.find(id);
        return questionMapper.toDto(a);
    }

    /**
     * Retrieve a list of AnswerDto objects which belong to question with questionId
     *
     * @param questionId the ID of the question for which to retrieve answers
     * @return a List of AnswerDto objects
     */
    public List<AnswerDto> getQuestionAnswers(long questionId) {
        return answerMapper.toDtoList(answerService.findAllByQuestionId(questionId));
    }

    /**
     * Create a new question
     *
     * @param dto a QuestionCreateDto object representing the new question to add
     * @return a QuestionDto object representing the added question
     */
    public QuestionDto create(QuestionCreateDto dto) {
        Question question = questionMapper.fromCreateDto(dto);

        Exercise exercise;
        exercise = exerciseService.find(dto.getExerciseId());

        exercise.getQuestions().add(question);
        question.setExercise(exercise);

        var answerDtos = dto.getAnswers();
        var answers = new HashSet<Answer>();
        for (var answerDto : answerDtos) {
            Answer answer = answerMapper.fromCreateDto(answerDto);
            answer = answerService.create(answer);
            answers.add(answer);
        }

        question.setAnswers(answers);
        var createdQuestion = questionService.create(question);

        for (var answer : answers) {
            answer.setQuestion(createdQuestion);
        }
        var result = questionMapper.toDto(createdQuestion);
        result.setExerciseId(exercise.getId());
        return result;
    }

    /**
     * Update question
     *
     * @param dto dto of updated question with correct id
     * @return dto of updated question
     */
    public QuestionDto update(long id, QuestionUpdateDto dto) {
        Question question = questionMapper.fromUpdateDto(dto);
        question.setId(id);
        var qston = questionService.find(id);
        List<Answer> questionAnswers = answerService.findAllByQuestionId(id);
        question.setAnswers(new HashSet<>(questionAnswers));

        question.setExercise(qston.getExercise());
        Question updatedQuestion = questionService.update(question);


        return questionMapper.toDto(updatedQuestion);
    }

    /**
     * Delete question
     *
     * @param id of qustion to delete
     */
    public void delete(long id) {
        var question = questionService.find(id);
        for (var answer : question.getAnswers()) {
            answerService.delete(answer.getId());
        }
        questionService.delete(id);
    }

    //@TestOnly
    public void reset() {
        questionService.reset();
        questionService.resetId();
    }

    public void resetTable() {
        questionService.reset();
        questionService.resetId();
    }
}
