package org.fuseri.moduleexercise.exercise;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.common.Result;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
import org.fuseri.model.dto.exercise.ExerciseDto;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;


@SpringBootTest
@AutoConfigureMockMvc
public class ExerciseTest {
    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    private MockMvc mockMvc;

    public static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private ExerciseCreateDto exercise1;
    private ExerciseCreateDto exercise2;
    private ExerciseCreateDto exercise3;


    @BeforeEach
    void init() {
        exercise1 = new ExerciseCreateDto("idioms", "exercise on basic idioms", 2, 0);
        exercise2 = new ExerciseCreateDto("idioms1", "exercise on intermediate idioms", 2, 0);
        exercise3 = new ExerciseCreateDto("idioms2", "exercise on basic idioms", 1, 0L);
        try {
            mockMvc.perform(post("/exercises").content(asJsonString(exercise1)).contentType(MediaType.APPLICATION_JSON));
            mockMvc.perform(post("/exercises").content(asJsonString(exercise2)).contentType(MediaType.APPLICATION_JSON));
            mockMvc.perform(post("/exercises").content(asJsonString(exercise3)).contentType(MediaType.APPLICATION_JSON));
        } catch (Exception e) {
            assert (false);
        }
    }

    @AfterEach
    void cleanup() {

        try {
            mockMvc.perform(delete("/questions/reset"));
            mockMvc.perform(delete("/exercises/reset"));
            mockMvc.perform(delete("/answers/reset"));
            } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    @Test
    void getExercise() {
        long id = 1L;
        try {
            var theId = String.format("/exercises/%s", id);
            var smth = mockMvc.perform(get(theId));
            smth.andExpect(status().isOk())
                    .andExpect(jsonPath("$.name", is(exercise1.getName())))
                    .andExpect(jsonPath("$.description", is(exercise1.getDescription())))
                    .andExpect(jsonPath("$.courseId", is((int) exercise1.getCourseId())))
                    .andExpect(jsonPath("$.difficulty", is(exercise1.getDifficulty())));
        } catch (Exception e) {
            //do absolutely nothing
        }
    }

    @Test
    void deleteExercise() {
        long id = 1L;
        try {
            var theId = String.format("/exercises/%s", id);
            var smth = mockMvc.perform(delete(theId));
            smth.andExpect(status().isNoContent());
        } catch (Exception e) {
            //do absolutely nothing
        }
    }

    @Test
    void deleteExercise_notFound() {
        long id = 999999L;
        try {
            var theId = String.format("/exercises/%s", id);
            var smth = mockMvc.perform(delete(theId));
            smth.andExpect(status().isNoContent());
        } catch (Exception e) {
            //do absolutely nothing
        }
    }


    @Test
    void getExercise_notFound() {
        long id = 999999L;
        try {
            var theId = String.format("/exercises/%s", id);
            var smth = mockMvc.perform(get(theId));
            smth.andExpect(status().isNotFound());
        } catch (Exception e) {
            //do absolutely nothing
        }
    }


    @Test
    void FindAll() {
        try {
            var dis = mockMvc.perform(get("/exercises").param("page", "0"));

            dis.andExpect(status().isOk())
                    .andExpect(jsonPath("$.total", is(3)))
                    .andExpect(jsonPath("$.items[0].name", is(exercise1.getName())))
                    .andExpect(jsonPath("$.items[0].description", is(exercise1.getDescription())))
                    .andExpect(jsonPath("$.items[0].difficulty", is(exercise1.getDifficulty())))
                    .andExpect(jsonPath("$.items[0].courseId", is((int) exercise1.getCourseId())))
                    .andExpect(jsonPath("$.items[1].name", is(exercise2.getName())))
                    .andExpect(jsonPath("$.items[1].description", is(exercise2.getDescription())))
                    .andExpect(jsonPath("$.items[1].difficulty", is(exercise2.getDifficulty())))
                    .andExpect(jsonPath("$.items[1].courseId", is((int) exercise2.getCourseId())))
                    .andExpect(jsonPath("$.items[2].name", is(exercise3.getName())))
                    .andExpect(jsonPath("$.items[2].description", is(exercise3.getDescription())))
                    .andExpect(jsonPath("$.items[2].difficulty", is(exercise3.getDifficulty())))
                    .andExpect(jsonPath("$.items[2].courseId", is((int) exercise3.getCourseId())));

        } catch (Exception e) {
            throw new RuntimeException(e);
        }


    }

    @Test
    void getFiltered() {

        try {
            var filtered = mockMvc.perform(get("/exercises/filter").param("page", "0").param("courseId", "0").param("difficulty", "2"));
            var content = filtered.andReturn().getResponse().getContentAsString();
            var res = objectMapper.readValue(content, new TypeReference<Result<ExerciseDto>>() {
            });

            assert (res.getTotal() == 2);
        } catch (Exception e) {
            assert (false);
        }
    }

//    @Test
//    void getByExercise() throws Exception {
//        var theId = String.format("/questions/exercise/%s", 9);
//
//        var smth = mockMvc.perform(get(theId).param("page", "0"));
//
//        var content = smth.andReturn().getResponse().getContentAsString();
//
//        var res = objectMapper.readValue(content, new TypeReference<Result<QuestionDto>>() {
//        });
//
//
//
////        assert (res.getItems().get(0).equals(question));
//    }

    @Test
    void testCreateExercise() throws Exception {
        var postExercise = new ExerciseCreateDto("idioms", "exercise on basic idioms", 2, 0L);

        mockMvc.perform(post("/exercises").content(asJsonString(postExercise)).contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.name").value("idioms"))
                .andExpect(jsonPath("$.description").value("exercise on basic idioms"))
                .andExpect(jsonPath("$.difficulty").value(2))
                .andExpect(jsonPath("$.courseId").value("0")).andReturn().getResponse().getContentAsString();
    }

    @Test
    void testCreateExerciseEmptyBody() throws Exception {
        var postExercise = "";

        mockMvc.perform(post("/exercises").content((postExercise)).contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError()).andReturn().getResponse().getContentAsString();
    }


    @Test
    void testCreateExerciseMissingDesc() throws Exception {
        var postExercise = """
                {
                  "name": "idioms",
                  "difficulty": 2,
                  "courseId": 0,
                }
                """;

        mockMvc.perform(post("/exercises").content((postExercise)).contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError()).andReturn().getResponse().getContentAsString();
    }

    @Test
    void testCreateExerciseMissingName() throws Exception {
        var postExercise = """
                {
                  "description: "exercise on basic idioms",
                  "difficulty": 2,
                  "courseId": 0,
                }
                """;

        mockMvc.perform(post("/exercises").content((postExercise)).contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError()).andReturn().getResponse().getContentAsString();
    }

    @Test
    void testCreateExerciseMissingDifficulty() throws Exception {
        var postExercise = """
                {
                  "name": "idioms"
                  "description: "exercise on basic idioms",
                  "courseId": 0
                }
                """;

        mockMvc.perform(post("/exercises").content((postExercise)).contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError()).andReturn().getResponse().getContentAsString();
    }

    @Test
    void testCreateExerciseMissingCourseId() throws Exception {
        var postExercise = """
                {
                  "name": "idioms"
                  "description: "exercise on basic idioms",
                  "difficulty": 0
                }
                """;

        mockMvc.perform(post("/exercises").content((postExercise)).contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().is4xxClientError()).andReturn().getResponse().getContentAsString();
    }


    @Test
    void testUpdate() {
        var postExercise = new ExerciseCreateDto("idioms", "exercise on basic idioms", 2, 0L);

        long id = 1L;

        try {
            var dis = mockMvc.perform(post("/exercises").content(asJsonString(postExercise)).contentType(MediaType.APPLICATION_JSON));
            var ok = dis.andReturn().getResponse().getContentAsString();

            var ll = objectMapper.readValue(ok, ExerciseDto.class);

            id = ll.getId();
        } catch (Exception e) {
            assert (false);
        }

        var expectedExercise = new ExerciseDto();
        expectedExercise.setId(id);
        expectedExercise.setName("idioms");
        expectedExercise.setDifficulty(2);
        expectedExercise.setCourseId(0L);
        expectedExercise.setDescription("exercise on basic idioms");

        var content = """
                {
                  "name": "idioms",
                  "description": "exercise on basic idioms",
                  "difficulty": 2,
                  "courseId": 0
                }
                """;


        try {
            var theId = String.format("/exercises/%d", id);
            var dis = mockMvc.perform(put(theId).content(content).contentType(MediaType.APPLICATION_JSON));

            var str = dis.andReturn().getResponse().getContentAsString();

            var res = objectMapper.readValue(str, ExerciseDto.class);

            assert res.equals(expectedExercise);

        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    @Test
    void testUpdateNotFound() {
        long id = 999999L;
        var content = """
                {
                  "name": "idioms",
                  "description": "exercise on basic idioms",
                  "difficulty": 2,
                  "courseId": 0
                }
                """;
        try {
            var theId = String.format("/exercises/%d", id);
            mockMvc.perform(put(theId).content(content).contentType(MediaType.APPLICATION_JSON))
                    .andExpect(status().isNotFound());

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    void testUpdateIncorrectBody() {
        long id = 1L;
        var content = """
                {
                  "description": "exercise on basic idioms",
                  "difficulty": 2,
                  "courseId": 0
                }
                """;
        try {
            var theId = String.format("/exercises/%d", id);
            mockMvc.perform(put(theId).content(content).contentType(MediaType.APPLICATION_JSON))
                    .andExpect(status().is4xxClientError());

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
